from argparse import ArgumentParser
from typing import Optional

from management_commands.utils import underscore


class Command:
    name: Optional[str] = None

    def get_command_name(self):
        """
        :return: name if specified in `self.name`, else generate from class name
        """
        if self.name is not None:
            return self.name

        return underscore(self.__class__.__name__)

    def add_subparser(self, subparsers):
        parser = subparsers.add_parser(self.get_command_name())
        self.add_arguments(parser=parser)
        parser.set_defaults(___handler=self.handle)

    def add_arguments(self, parser: ArgumentParser):
        pass

    def handle(self, **kwargs) -> None:
        raise NotImplementedError



from typing import List
import argparse

from management_commands.command import Command


def main(
        commands: List[Command],
        **kwargs,
) -> None:
    parser = argparse.ArgumentParser(**kwargs)
    subparsers = parser.add_subparsers(title='commands')

    for command_obj in commands:
        command_obj.add_subparser(subparsers)

    args = vars(parser.parse_args())

    if '___handler' not in args:
        parser.print_usage()
        parser.exit(1)

    handler = args.pop('___handler')
    handler(**args)



from setuptools import setup


VERSION = "0.3"

setup(
    name='management_commands',
    version=VERSION,
    url='https://github.com/a1fred/management_commands',
    license='MIT',
    packages=['management_commands', ],
    author='a1fred',
    author_email='demalf@gmail.com',
    description='Extra small zero-dependency management commands library for python apps',
    classifiers=[
        'Environment :: Console',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
    ],
)
